@@ -57,6 +57,8 @@ AcquirePasswordDialog::slotOk ()
 	emit sendPassphrase (_net);
 
 	notify = new ActivationStageNotifyNetwork (_net->getEssid(), _ctx->getTray (), "ActivationStageNotify", false, 0, _ctx, _dev );
+	notify->setAnchor(_ctx->getTray()->getAnchor());
+	notify->show();
 	KDialogBase::slotOk();
 }
 
@@ -130,59 +132,89 @@ AcquirePasswordDialog::comboEncryptionFill ()
 		bool wpa1_available = capabilities & NM_802_11_CAP_PROTO_WPA;
 		bool wpa2_available = capabilities & NM_802_11_CAP_PROTO_WPA2;
 
-		comboEnc->insertItem (i18n ("WPA Personal"), id);
-		_id_wpa_wpa2personal = id++;
-		comboEnc->insertItem (i18n ("WPA Enterprise"), id);
-		_id_wpa_wpa2enterprise = id;
+		if (capabilities & NM_802_11_CAP_KEY_MGMT_802_1X) {
+			// we have a WPA-EAP network here
+			comboEnc->insertItem (i18n ("WPA Enterprise"), id);
+			_id_wpa_wpa2enterprise = id;
 
-		_encryptionMap [_id_wpa_wpa2personal] = new EncryptionWPAPersonal ();
-		_encryptionMap [_id_wpa_wpa2enterprise] = new EncryptionWPAEnterprise ();
+			_encryptionMap [_id_wpa_wpa2enterprise] = new EncryptionWPAEnterprise ();
+			EncryptionWPAEnterprise* encEnterprise = (EncryptionWPAEnterprise*) _encryptionMap [_id_wpa_wpa2enterprise];
 
-		//EncryptionWPAPersonal* enc = (EncryptionWPAPersonal*) _encryptionMap [cryptoWidget->comboEncryption->currentItem ()];
-		EncryptionWPAPersonal* encPersonal = (EncryptionWPAPersonal*) _encryptionMap [_id_wpa_wpa2personal];
-		EncryptionWPAEnterprise* encEnterprise = (EncryptionWPAEnterprise*) _encryptionMap [_id_wpa_wpa2enterprise];
+			/* Let the user choose if WPA1 and WPA2 are both available. Else: Disable radio buttons */
+			cryptoWidget->EAPrbWPA2->setEnabled (wpa1_available && wpa2_available);
+			cryptoWidget->EAPrbWPA1->setEnabled (wpa1_available && wpa2_available);
 
-		comboProto->insertItem (i18n ("Automatic"), protoId);
-		_id_wpa_personal_auto = protoId++;
+			if (wpa1_available && !wpa2_available) {
+				encEnterprise->setVersion (WPA1);
+				cryptoWidget->EAPrbWPA1->setChecked (true);
+				cryptoWidget->EAPrbWPA2->setChecked (false);
+			} else if (!wpa1_available && wpa2_available) {
+				encEnterprise->setVersion (WPA2);
+				cryptoWidget->EAPrbWPA2->setChecked (true);
+				cryptoWidget->EAPrbWPA1->setChecked (false);
+			}
+			protoId = 0;
+			cryptoWidget->cboWPAEAPprotocol->insertItem (i18n ("Automatic"), protoId);
+			_id_wpa_eap_auto = protoId++;
 
-		if (capabilities & NM_802_11_CAP_CIPHER_TKIP) {
-			comboProto->insertItem (i18n ("Temporal Key Integrity Protocol (TKIP)"), protoId);
-			_id_wpa_personal_tkip = protoId++;
-		}
-		if (capabilities & NM_802_11_CAP_CIPHER_CCMP) {
-			comboProto->insertItem (i18n ("AES-Counter Mode CBC-MAC Protocol (AES-CCMP)"), protoId);
-			_id_wpa_personal_ccmp = protoId++;
+			if (capabilities & NM_802_11_CAP_CIPHER_TKIP) {
+				cryptoWidget->cboWPAEAPprotocol->insertItem (i18n ("Temporal Key Integrity Protocol (TKIP)"), protoId);
+				_id_wpa_eap_tkip = protoId++;
+			}
+			if (capabilities & NM_802_11_CAP_CIPHER_CCMP) {
+				cryptoWidget->cboWPAEAPprotocol->insertItem (i18n ("AES-Counter Mode CBC-MAC Protocol (AES-CCMP)"), protoId);
+				_id_wpa_eap_ccmp = protoId++;
+			}
 		}
 
-		/* Let the user choose if WPA1 and WPA2 are both available. Else: Disable radio buttons */
-		cryptoWidget->radioButtonWPA2->setEnabled (wpa1_available && wpa2_available);
-		cryptoWidget->radioButtonWPA1->setEnabled (wpa1_available && wpa2_available);
-		cryptoWidget->EAPrbWPA2->setEnabled (wpa1_available && wpa2_available);
-		cryptoWidget->EAPrbWPA1->setEnabled (wpa1_available && wpa2_available);
+		if (capabilities & NM_802_11_CAP_KEY_MGMT_PSK) {
+			// we have a WPA-PSK network here
+			comboEnc->insertItem (i18n ("WPA Personal"), id);
+			_id_wpa_wpa2personal = id++;
 
-		if (wpa1_available && !wpa2_available) {
-			encPersonal->setVersion (WPA1);
-			encEnterprise->setVersion (WPA1);
-			cryptoWidget->radioButtonWPA1->setChecked (true);
-			cryptoWidget->radioButtonWPA2->setChecked (false);
-			cryptoWidget->EAPrbWPA1->setChecked (true);
-			cryptoWidget->EAPrbWPA2->setChecked (false);
-		} else if (!wpa1_available && wpa2_available) {
-			encPersonal->setVersion (WPA2);
-			encEnterprise->setVersion (WPA2);
-			cryptoWidget->radioButtonWPA2->setChecked (true);
-			cryptoWidget->radioButtonWPA1->setChecked (false);
-			cryptoWidget->EAPrbWPA2->setChecked (true);
-			cryptoWidget->EAPrbWPA1->setChecked (false);
+			_encryptionMap [_id_wpa_wpa2personal] = new EncryptionWPAPersonal ();
+			EncryptionWPAPersonal* encPersonal = (EncryptionWPAPersonal*) _encryptionMap [_id_wpa_wpa2personal];
+
+			protoId = 0;
+
+			comboProto->insertItem (i18n ("Automatic"), protoId);
+			_id_wpa_personal_auto = protoId++;
+
+			if (capabilities & NM_802_11_CAP_CIPHER_TKIP) {
+				comboProto->insertItem (i18n ("Temporal Key Integrity Protocol (TKIP)"), protoId);
+				_id_wpa_personal_tkip = protoId++;
+			}
+			if (capabilities & NM_802_11_CAP_CIPHER_CCMP) {
+				comboProto->insertItem (i18n ("AES-Counter Mode CBC-MAC Protocol (AES-CCMP)"), protoId);
+				_id_wpa_personal_ccmp = protoId++;
+			}
+
+			/* Let the user choose if WPA1 and WPA2 are both available. Else: Disable radio buttons */
+			cryptoWidget->radioButtonWPA2->setEnabled (wpa1_available && wpa2_available);
+			cryptoWidget->radioButtonWPA1->setEnabled (wpa1_available && wpa2_available);
+		
+			if (wpa1_available && !wpa2_available) {
+				encPersonal->setVersion (WPA1);
+				cryptoWidget->radioButtonWPA1->setChecked (true);
+				cryptoWidget->radioButtonWPA2->setChecked (false);
+			} else if (!wpa1_available && wpa2_available) {
+				encPersonal->setVersion (WPA2);
+				cryptoWidget->radioButtonWPA2->setChecked (true);
+				cryptoWidget->radioButtonWPA1->setChecked (false);
+			}
+	
+			/* Prefer NM_AUTH_TYPE_WPA_PSK_AUTO */
+			comboProto->setCurrentItem (_id_wpa_personal_auto);
 		}
 
-		comboEnc->setCurrentItem (_id_wpa_wpa2personal);
-		/* Prefer NM_AUTH_TYPE_WPA_PSK_AUTO */
-		comboProto->setCurrentItem (_id_wpa_personal_auto);
+		if (_id_wpa_wpa2personal > 0)
+			comboEnc->setCurrentItem (_id_wpa_wpa2personal);
+		else if (_id_wpa_wpa2enterprise > 0)
+			comboEnc->setCurrentItem (_id_wpa_wpa2enterprise);
 	}
 }
 
-AcquirePasswordDialog::AcquirePasswordDialog (QWidget* parent, const char *name, bool modal, WFlags fl, KNetworkManager* ctx, QString obj_path, QString net_path, QString essid)
+AcquirePasswordDialog::AcquirePasswordDialog (QWidget* parent, const char *name, bool modal, WFlags fl, KNetworkManager* ctx, QString obj_path, QString net_path, QString essid, QString message)
                             : WirelessDialog (parent, name, modal, fl, ctx), _obj_path(obj_path), _net_path(net_path), _essid(essid)
 {
 	delete _net; // Free the object created in WirelessDialog constructor
@@ -192,6 +224,13 @@ AcquirePasswordDialog::AcquirePasswordDialog (QWidget* parent, const char *name,
 	_net = _dev->getNetwork (net_path);
 
 	QWidget * mainWid = mainWidget();
+
+	if (!message.isEmpty())
+	{
+		QLabel * label = new QLabel(mainWid);
+		label->setText("<b>" + message + "</b>");
+	}
+
 	NetworkLabelWidget * labelWidget = new NetworkLabelWidget( mainWid );
 	labelWidget->lblEssidVal->setText (essid);
 	labelWidget->lblDeviceVal->setText ( QString("%1 %2 (%3)")
@@ -210,15 +249,17 @@ AcquirePasswordDialog::AcquirePasswordDialog (QWidget* parent, const char *name,
 	
 	connect (store, SIGNAL (deviceStoreChanged (DeviceStore *)),
 		 this, SLOT (slotDeviceStoreChanged (DeviceStore *)));
-		 
+/*		 
 	connect (this, SIGNAL (sendPassphrase (Network*)),
 		 _ctx->getNetworkManagerInfo (), SLOT (sendPassphrase (Network*)));
 
 	connect (this, SIGNAL (cancelClicked ()),
 		 _ctx->getNetworkManagerInfo (), SLOT (sendPassphraseError ()));
-
+*/
 	connect (this, SIGNAL (cancelClicked ()),
 		 this, SLOT   (invalidateSecretMap ()));
+
+	updateModel();
 }
 
 AcquirePasswordDialog::~AcquirePasswordDialog ()
@@ -231,14 +272,44 @@ AcquirePasswordDialog::~AcquirePasswordDialog ()
 void
 NewWirelessNetworkDialog::slotOk()
 {
+	bool new_network = true;
 	updateModel();
 
-	_dev->setPendingNetwork (_net);
+	/*
+	  if the network with the entered essid is already in our list
+	  we use the one in our list instead of the newly created one.
+	  But we have to transfer the encryption object.
+	*/
+	NetworkList nets = _dev->getNetworkList();
+	for (NetworkList::Iterator it = nets.begin(); it != nets.end(); ++it) {
+		if ((*it)->getEssid() == _net->getEssid()) {
+			// move encryption object to the network in the list
+			(*it)->setEncryption(_net->getEncryption());
+			_net->setEncryption(NULL);
+			// next steps should take our list network
+			delete _net;
+			_net = (*it);
+			// the network is known -> not new
+			new_network = false;
+			break;
+		}
+	}
+
+	/*
+	  If the network is already in our list do not set is as pending!
+	  Instead clear the current pending network because this is no more pending.
+	*/
+	if (new_network)
+		_dev->setPendingNetwork (_net);
+	else
+		_dev->clearPendingNetwork ();
 
 	emit activateNetwork (_net, _dev);
 
 	ActivationStageNotifyNetwork* notify = NULL;
-	notify = new ActivationStageNotifyNetwork ( _net->getEssid(), this, "ActivationStageNotify", true, 0, _ctx, _dev );
+	notify = new ActivationStageNotifyNetwork ( _net->getEssid(), _ctx->getTray(), "ActivationStageNotify", true, 0, _ctx, _dev );
+	notify->setAnchor(_ctx->getTray()->getAnchor());
+	notify->show();
 	KDialogBase::slotOk();
 }
 
@@ -284,6 +355,16 @@ NewWirelessNetworkDialog::comboEncryptionFill ()
 	_id_wpa_personal_tkip = id++;
 	proto->insertItem (i18n ("AES-Counter Mode CBC-MAC Protocol (AES-CCMP)"), id);
 	_id_wpa_personal_ccmp = id++;
+
+	id = 0;
+	proto = cryptoWidget->cboWPAEAPprotocol;
+	proto->insertItem (i18n ("Automatic"), id);
+	_id_wpa_eap_auto = id++;
+	proto->insertItem (i18n ("Temporal Key Integrity Protocol (TKIP)"), id);
+	_id_wpa_eap_tkip = id++;
+	proto->insertItem (i18n ("AES-Counter Mode CBC-MAC Protocol (AES-CCMP)"), id);
+	_id_wpa_eap_ccmp = id++;
+
 }
 
 void
@@ -432,6 +513,22 @@ WirelessDialog::radioButtonWPA2_toggled (bool on)
 }
 
 /* WPA Enterprise */
+void
+WirelessDialog::comboWPAEAPProtocol_activated (int id)
+{
+	WPAProtocol proto = WPA_AUTO;
+	EncryptionWPAEnterprise* enc = (EncryptionWPAEnterprise*) _encryptionMap [_id_wpa_wpa2enterprise];
+
+	if (id == _id_wpa_eap_auto)
+		proto = WPA_AUTO;
+	else if (id == _id_wpa_eap_tkip)
+		proto = WPA_TKIP;
+	else if (id == _id_wpa_eap_ccmp)
+		proto = WPA_CCMP_AES;
+
+	enc->setProtocol (proto);
+}
+
 
 void
 WirelessDialog::groupWPAEAPAdvanced_toggled (bool on)
@@ -442,6 +539,7 @@ WirelessDialog::groupWPAEAPAdvanced_toggled (bool on)
 	if (!on) {
 		enc->setDefaults ();
 		cryptoWidget->EAPcomboMethod->setCurrentItem (0);
+		cryptoWidget->EAPcomboPhaseTwo->setCurrentItem (0);
 		cryptoWidget->EAPrbWPA1->setChecked (true);
 	}
 }
@@ -480,13 +578,75 @@ WirelessDialog::EAPcomboMethod_activated (int id)
 		case 2:
 			method = EAP_TTLS;
 			break;
+		case 3:
+			method = EAP_LEAP;
+			enc->setLeapMethod ("IEEE8021X");
+			break;
+		case 4:
+			method = EAP_LEAP;
+			enc->setLeapMethod ("WPA-EAP");
+			break;
 		default:
 			method = EAP_PEAP;
 	}
+
+	// fields not needed for LEAP/IEEE8021X
+	if(method == EAP_LEAP && enc->getLeapMethod() == "IEEE8021X")
+	{
+		cryptoWidget->EAPleditAnonIdentity->setDisabled(method == EAP_LEAP);
+		cryptoWidget->EAPleditPrivatePassword->setDisabled(method == EAP_LEAP);
+		cryptoWidget->EAPurlClient->setDisabled(method == EAP_LEAP);
+		cryptoWidget->EAPurlPrivate->setDisabled(method == EAP_LEAP);
+		cryptoWidget->EAPurlCA->setDisabled(method == EAP_LEAP);
+		cryptoWidget->EAPleditPassword->setEnabled(method == EAP_LEAP);
+	}
+	else
+	{
+		// disable unused fields
+		cryptoWidget->EAPleditPrivatePassword->setDisabled(method == EAP_PEAP);
+		cryptoWidget->EAPurlPrivate->setDisabled(method == EAP_PEAP);
+		cryptoWidget->EAPurlClient->setEnabled(method == EAP_PEAP || method == EAP_TLS || method == EAP_TTLS);
+		cryptoWidget->EAPurlCA->setEnabled(method == EAP_PEAP || EAP_TLS);
+		cryptoWidget->EAPurlClient->setEnabled(method == EAP_TLS);
+		cryptoWidget->EAPurlPrivate->setEnabled(method == EAP_TLS);
+		cryptoWidget->EAPleditPrivatePassword->setEnabled(method == EAP_TLS);
+		cryptoWidget->EAPleditPassword->setEnabled(method == EAP_PEAP || method == EAP_TTLS || (method == EAP_LEAP && enc->getLeapMethod() == "WPA-EAP") );
+		cryptoWidget->EAPleditAnonIdentity->setEnabled(method == EAP_PEAP || method == EAP_TTLS || (method == EAP_LEAP && enc->getLeapMethod() == "WPA-EAP") );
+	}
+
 	
 	enc->setMethod (method);
 }
 
+void
+WirelessDialog::EAPcomboPhaseTwo_activated (int id)
+{
+	EAPPhaseTwo type = PHASE2_NONE;
+	EncryptionWPAEnterprise* enc = (EncryptionWPAEnterprise*) _encryptionMap [_id_wpa_wpa2enterprise];
+
+	switch (id) {
+		case 0:
+			type = PHASE2_NONE;
+			break;
+		case 1:
+			type = PHASE2_PAP;
+			break;
+		case 2:
+			type = PHASE2_MSCHAP;
+			break;
+		case 3:
+			type = PHASE2_MSCHAPV2;
+			break;
+		case 4:
+			type = PHASE2_GTC;
+			break;
+		default:
+			type = PHASE2_NONE;
+	}
+
+	enc->setPhaseTwoAuth (type);
+}
+
 void
 WirelessDialog::EAPleditIdentity_textChanged (const QString & identity)
 {
@@ -501,6 +661,7 @@ WirelessDialog::EAPleditPassword_textChanged (const QString & identityPassword)
 	SecretMap map = enc->getSecrets();
 	map.insert( EncryptionWPAEnterprise::IdPasswordKey, identityPassword );
 	enc->setSecrets (map);
+	updateModel();
 }
 
 void
@@ -539,6 +700,7 @@ WirelessDialog::EAPleditPrivatePassword_textChanged (const QString & certPrivate
 	SecretMap map = enc->getSecrets();
 	map.insert( EncryptionWPAEnterprise::CertPrivatePasswordKey, certPrivatePassword );
 	enc->setSecrets (map);
+	updateModel();
 }
 	
 void
@@ -578,6 +740,8 @@ WirelessDialog::connectCryptoWidget()
 		 this,                           SLOT   (radioButtonWPA2_toggled (bool)));
 
 	/* WPA Enterprise */
+	connect (cryptoWidget->cboWPAEAPprotocol,       SIGNAL(activated(int)),
+		 this,                                  SLOT(comboWPAEAPProtocol_activated(int)));
 	connect (cryptoWidget->groupEncAdvanced,        SIGNAL (toggled (bool)),
 		 this,                                  SLOT   (groupWPAEAPAdvanced_toggled (bool)));
 	connect (cryptoWidget->EAPcbShow,               SIGNAL (toggled (bool)),
@@ -588,6 +752,8 @@ WirelessDialog::connectCryptoWidget()
 		 this,                                  SLOT   (EAPrbWPA2_toggled (bool)));
 	connect (cryptoWidget->EAPcomboMethod,          SIGNAL (activated (int)),
 		 this,                                  SLOT   (EAPcomboMethod_activated (int)));
+	connect (cryptoWidget->EAPcomboPhaseTwo,        SIGNAL (activated (int)),
+		 this,                                  SLOT   (EAPcomboPhaseTwo_activated (int)));
 	connect (cryptoWidget->EAPleditIdentity,        SIGNAL (textChanged (const QString &)),
 		 this,                                  SLOT   (EAPleditIdentity_textChanged (const QString &)));
 	connect (cryptoWidget->EAPleditPassword,        SIGNAL (textChanged (const QString &)),
@@ -602,6 +768,18 @@ WirelessDialog::connectCryptoWidget()
 		 this,                                  SLOT   (EAPurlPrivate_textChanged (const QString &)));
 	connect (cryptoWidget->EAPleditPrivatePassword, SIGNAL (textChanged (const QString &)),
 		 this,                                  SLOT   (EAPleditPrivatePassword_textChanged (const QString &)));
+
+#ifndef HAVE_PHASE2_AUTH
+	// no phase2 -> disable the appropriate combobox
+	cryptoWidget->EAPcomboPhaseTwo->setDisabled(true);
+	cryptoWidget->textLabel1->setDisabled(true);
+#endif
+
+#ifndef HAVE_LEAP
+	// no leap -> remove the appropriate entries from the combobox
+	cryptoWidget->EAPcomboMethod->removeItem(4);
+	cryptoWidget->EAPcomboMethod->removeItem(3);
+#endif
 }
 
 void
@@ -629,6 +807,7 @@ WirelessDialog::comboEncryption_activated (int id)
 	} else if (id == _id_wpa_wpa2enterprise) {
 		kdDebug() << k_funcinfo << "WPA Enterprise" << endl;
 		cryptoWidget->stackEncAdvanced->raiseWidget(cryptoWidget->advancedWPAEnterpriseWidget);
+		EAPcomboMethod_activated(cryptoWidget->EAPcomboMethod->currentItem());
 	}
 
 	/* WPA Enterprise does not use lblSecret, leditSecret and cbShow */
@@ -687,6 +866,7 @@ WirelessDialog::WirelessDialog(QWidget * parent, const char *name, bool modal, W
 	_net = new Network ();
 	makeVBoxMainWidget();
 	enableButton (KDialogBase::Ok, false);
+
 }
 
 WirelessDialog::~WirelessDialog()
