@@ -33,12 +33,6 @@
 #include "knetworkmanager-storage.h"
 #include <NetworkManager/dbus-helpers.h>
 
-void
-Encryption::addCipher (IEEE_802_11_Cipher* cipher)
-{
-	_cipherList->append (cipher);
-}
-
 bool
 Encryption::isValid (const QString & essid)
 {
@@ -81,21 +75,54 @@ Encryption::setSecrets (const SecretMap & secret)
 void
 Encryption::setNetwork (Network* network)
 {
-	_network = network; 
+	_network = network;
 	_dirty = true;
 }
 
 bool
 Encryption::hasStoredKey (void) const
 {
-	return _keyStored;
+	return KNetworkManagerStorage::getInstance()->hasCredentialsStored(_network->getEssid());
+}
+
+void
+Encryption::slotCredentialsLoaded (QString /*id*/, QMap<QString, QString> map, bool canceled)
+{
+	bool result = false;
+
+	if ( !map.isEmpty() ) {
+		_secret = map;
+		result = true;
+	}
+	emit keyRestored(result, canceled);
+
+	// we got the desired information -> delete the request
+	delete _credentialsRequest;
+	_credentialsRequest = NULL;
+}
+
+void
+Encryption::restoreKeyAsync (void)
+{
+	if ( hasStoredKey() && _network) {
+		// if a credentialrequest for this encryption is already running, simply delete it
+		if (_credentialsRequest) delete _credentialsRequest;
+
+		// lets start a new credential request
+		_credentialsRequest = KNetworkManagerStorage::getInstance( )->credentialsAsync(_network->getEssid());
+		connect(_credentialsRequest, SIGNAL(credentialsLoaded(QString, QMap<QString, QString>, bool)), this, SLOT(slotCredentialsLoaded (QString, QMap<QString, QString>, bool)));
+		_credentialsRequest->loadCredentials();
+	} else {
+		emit keyRestored(false, false);
+	}
 }
 
 bool
 Encryption::restoreKey (void)
 {
+	kdDebug() << k_funcinfo << endl;
 	bool result = false;
-	if ( _keyStored && _network) {
+	if ( hasStoredKey() && _network) {
 		SecretMap secret = KNetworkManagerStorage::getInstance( )->credentials( _network->getEssid( ) );
 
 		if ( !secret.isEmpty() ) {
@@ -109,10 +136,10 @@ Encryption::restoreKey (void)
 bool
 Encryption::persistKey (void) const
 {
-	if ( !_secret["password"].isEmpty() && _network )
-		_keyStored = KNetworkManagerStorage::getInstance( )->storeCredentials( _network->getEssid( ), _secret);
+	if ( !_secret.isEmpty() && _network )
+		return KNetworkManagerStorage::getInstance( )->storeCredentials( _network->getEssid( ), _secret);
 	
-	return _keyStored;
+	return hasStoredKey();
 }
 
 bool
@@ -134,16 +161,26 @@ Encryption::setWeCipher (int we_cipher)
 	_we_cipher = we_cipher;
 }
 
-Encryption::Encryption () : _currentCipher(0), _cipherList(0), _network(0)
+Encryption::Encryption () : _currentCipher(0), _cipherList(0), _network(0), _credentialsRequest(0)
 {
-	_keyStored = false;
 	_dirty = true;
 	_we_cipher = -1;
 }
 
+void Encryption::clearCipherList()
+{
+    if (_cipherList)
+        for (CipherList::Iterator it = _cipherList->begin(); it != _cipherList->end(); ++it)
+        {
+            ieee_802_11_cipher_unref(*it);
+            (*it) = NULL;
+        }
+    delete _cipherList;
+}
+
 Encryption::~Encryption ()
 {
-	delete _cipherList;
+    clearCipherList();
 }
 
 
@@ -185,6 +222,7 @@ EncryptionNone::setDefaults (void)
 }
 
 EncryptionNone::EncryptionNone ()
+    : Encryption()
 {
 
 }
@@ -194,12 +232,12 @@ EncryptionNone::~EncryptionNone ()
 
 }
 
-void EncryptionNone::persist( KConfigBase * cfg, bool withKey ) const
+void EncryptionNone::persist( KConfigBase * cfg, bool /*withKey*/ ) const
 {
 	cfg->writeEntry( "Encryption", "none" );
 }
 
-void EncryptionNone::restore( KConfigBase *, double version,  bool withKey)
+void EncryptionNone::restore( KConfigBase *, const char* /*version*/,  bool /*withKey*/)
 {
 	kdDebug() << k_funcinfo << endl;
 }
@@ -220,7 +258,7 @@ EncryptionWEP::serialize (DBusMessage* msg, const QString & essid)
 	 * of the data to NM and supply a key on demand. This will show NM that we know
 	 * the network we are connecting to and there is no need to ask user for a new key.
 	 */
-	if ( _keyStored && _secret["password"].isEmpty() ) {
+	if ( hasStoredKey() && _secret["password"].isEmpty() ) {
 		if ( !msg || essid.isEmpty() )
 			return false;
 
@@ -288,18 +326,18 @@ EncryptionWEP::setType( WEPType type )
 {
 	_type = type;
 	_dirty = true;
-	delete _cipherList;
+	clearCipherList();
 	_cipherList = new CipherList ();
 	
 	if (_type == WEP_ASCII) {
-		addCipher (cipher_wep64_ascii_new  ());
-		addCipher (cipher_wep128_ascii_new ());
+		_cipherList->append (cipher_wep64_ascii_new  ());
+		_cipherList->append (cipher_wep128_ascii_new  ());
 	} else if (_type == WEP_HEX) {
-		addCipher (cipher_wep64_hex_new  ());
-		addCipher (cipher_wep128_hex_new ());
+		_cipherList->append (cipher_wep64_hex_new  ());
+		_cipherList->append (cipher_wep128_hex_new  ());
 	} else if (_type == WEP_PASSPHRASE) {
-		addCipher (cipher_wep64_passphrase_new  ());
-		addCipher (cipher_wep128_passphrase_new ());
+		_cipherList->append (cipher_wep64_passphrase_new  ());
+		_cipherList->append (cipher_wep128_passphrase_new  ());
 	}
 }
 
@@ -351,12 +389,11 @@ void EncryptionWEP::persist( KConfigBase * cfg, bool withKey ) const
 	}
 	if ( withKey )
 		this->persistKey( );
-	cfg->writeEntry( "KeyStored", _keyStored );
 	cfg->writeEntry ( "Cipher", _we_cipher );
 	_dirty = false;
 }
 
-void EncryptionWEP::restore( KConfigBase * cfg, double version, bool withKey)
+void EncryptionWEP::restore( KConfigBase * cfg, const char* version, bool withKey)
 {
 	kdDebug() << k_funcinfo << endl;
 	QString wepType = cfg->readEntry( "WEPType", "ASCII" );
@@ -377,15 +414,13 @@ void EncryptionWEP::restore( KConfigBase * cfg, double version, bool withKey)
 
 	_dirty = false;
 
-	if (version == 0) {
-		_keyStored = true; // set this to true, as it is most likely stored
-		_keyStored = this->restoreKey ();
-		if (_keyStored && _network)
+	if (strcmp(version, "0") == 0 ) {
+		bool keyStored = this->restoreKey ();
+		if (keyStored && _network)
 			this->isValid (_network->getEssid ()); // set _we_cipher
 		this->persist (cfg, true);
-	} else if (version == 0.1) {
-		_keyStored = cfg->readBoolEntry ("KeyStored", false);
-		if (withKey && _keyStored)
+	} else {
+		if (withKey && hasStoredKey())
 			this->restoreKey ();
 		_we_cipher = cfg->readNumEntry ("Cipher", -1);
 	}
@@ -406,10 +441,9 @@ EncryptionWPAPersonal::serialize (DBusMessage* msg, const QString & essid)
 	 * of the data to NM and supply a key on demand. This will show NM that we know
 	 * the network we are connecting to and there is no need to ask user for a new key.
 	 */
-	if ( _keyStored && _secret["password"].isEmpty() ) {
+	if ( hasStoredKey() && _secret["password"].isEmpty() ) {
 		if ( !msg || essid.isEmpty() )
 			return false;
-
 		/*
 		 * We only need to pass we_cipher remembered since the last connection.
 		 * Unfortunately, libnm-util functions accept only IEEE_802_11_Cipher object
@@ -509,8 +543,8 @@ EncryptionWPAPersonal::getVersion (void)
 EncryptionWPAPersonal::EncryptionWPAPersonal ()
 {
 	_cipherList = new CipherList (); 
-	addCipher (cipher_wpa_psk_hex_new ());
-	addCipher (cipher_wpa_psk_passphrase_new ());
+        _cipherList->append (cipher_wpa_psk_hex_new ());
+        _cipherList->append (cipher_wpa_psk_passphrase_new ());
 	setDefaults ();
 }
 
@@ -538,12 +572,11 @@ void EncryptionWPAPersonal::persist( KConfigBase * cfg, bool withKey ) const
 	if ( withKey )
 		this->persistKey( );
 
-	cfg->writeEntry ("KeyStored", _keyStored);
 	cfg->writeEntry ("Cipher", _we_cipher);
 	_dirty = false;
 }
 
-void EncryptionWPAPersonal::restore( KConfigBase * cfg, double version, bool withKey)
+void EncryptionWPAPersonal::restore( KConfigBase * cfg, const char* version, bool withKey)
 {
 	kdDebug() << k_funcinfo << endl;
 
@@ -564,15 +597,13 @@ void EncryptionWPAPersonal::restore( KConfigBase * cfg, double version, bool wit
 
 	_dirty = false;
 
-	if (version == 0) {
-		_keyStored = true; // set this to true, as it is most likely stored
-		_keyStored = this->restoreKey ();
-		if (_keyStored && _network)
+	if (strcmp(version, "0") == 0) {
+		bool keyStored = this->restoreKey ();
+		if (keyStored && _network)
 			this->isValid (_network->getEssid ()); // set _we_cipher
 		this->persist (cfg, true);
-	} else if (version == 0.1) {
-		_keyStored = cfg->readBoolEntry ("KeyStored", false);
-		if (withKey && _keyStored)
+	} else {
+		if (withKey && hasStoredKey())
 			this->restoreKey ();
 		_we_cipher = cfg->readNumEntry ("Cipher", -1);
 	}
@@ -590,12 +621,32 @@ EncryptionWPAEnterprise::serialize (DBusMessage* msg, const QString & essid)
 	kdDebug() << k_funcinfo << endl;
 	DBusMessageIter iter;
 	bool status = false;
-
+	int eap_method = _method;
+#ifdef HAVE_PHASE2_AUTH
+	eap_method |= _phaseTwo;
+#endif
 	int keyType = NM_AUTH_TYPE_WPA_PSK_AUTO;
 
-	if (!msg || !essid || !isValid( essid ) )
+	switch(_protocol)
+	{
+		case WPA_AUTO:
+			keyType = NM_AUTH_TYPE_WPA_PSK_AUTO;
+			break;
+		case WPA_TKIP:
+			keyType = IW_AUTH_CIPHER_TKIP;
+			break;
+		case WPA_CCMP_AES:
+			keyType = IW_AUTH_CIPHER_CCMP;
+			break;
+	}	
+	if (!msg || !essid )
 		return false;
 
+	if (! (hasStoredKey() && _secret[IdPasswordKey].length() == 0 && _secret[CertPrivatePasswordKey].length() == 0)) {
+		if (!isValid( essid ))
+			return false;
+	}
+
 	/* FIXME: If the user denies access to kwallet we'd send NULL over the bus.  Avoid that. */
 	if (!_secret[IdPasswordKey])
 		_secret[IdPasswordKey] = "";
@@ -609,11 +660,23 @@ EncryptionWPAEnterprise::serialize (DBusMessage* msg, const QString & essid)
 		      " version: " << _version << endl;
 
 	dbus_message_iter_init_append (msg, &iter);
-	status = nmu_security_serialize_wpa_eap_with_cipher (&iter, _method, keyType, _identity.utf8(),
-							     _secret[IdPasswordKey].utf8(), _anonIdentity.utf8(),
-							     _secret[CertPrivatePasswordKey].utf8(),
-							     _certPrivate.utf8(), _certClient.utf8(), _certCA.utf8(),
-							     _version);
+	if (_method == EAP_LEAP )
+	{
+#ifdef HAVE_LEAP
+		status = nmu_security_serialize_leap_with_cipher(&iter, _identity.utf8(), _secret[IdPasswordKey] ,
+									_leapmethod );
+#else
+		return false;
+#endif
+	}
+	else
+	{
+		status = nmu_security_serialize_wpa_eap_with_cipher (&iter, eap_method, keyType, _identity.utf8(),
+								     _secret[IdPasswordKey].utf8(), _anonIdentity.utf8(),
+								     _secret[CertPrivatePasswordKey].utf8(),
+								     _certPrivate.utf8(), _certClient.utf8(), _certCA.utf8(),
+								     _version);
+	}
 
 	return status;
 }
@@ -622,7 +685,10 @@ bool
 EncryptionWPAEnterprise::deserialize( DBusMessageIter * iter, int we_cipher )
 {
 	int method;
+	EAPMethod eapMethod;
+	EAPPhaseTwo eapPhaseTwo;
 	int keyType;
+	char* leapmethod;
 	char* identity = NULL;
 	char* identityPassword = NULL;
 	char* anonIdentity = NULL;
@@ -634,19 +700,46 @@ EncryptionWPAEnterprise::deserialize( DBusMessageIter * iter, int we_cipher )
 
 	if ( !iter )
 		return false;
-	if ( !(we_cipher == NM_AUTH_TYPE_WPA_EAP ))
-		return false;
 
-	if (!nmu_security_deserialize_wpa_eap (iter, &method, &keyType, &identity, &identityPassword, &anonIdentity,
-					       &certPrivatePassword, &certPrivate, &certClient, &certCA, &version))
+	if (method == EAP_LEAP)
+	{
+#ifdef HAVE_LEAP
+		if (we_cipher != NM_AUTH_TYPE_LEAP)
+			return false;
+		if (!nmu_security_deserialize_leap (iter, &identity, &identityPassword, &leapmethod))
+			return false;
+#else
 		return false;
+#endif
+	}
+	else
+	{
+		if (we_cipher != NM_AUTH_TYPE_WPA_EAP)
+			return false;
+
+		if (!nmu_security_deserialize_wpa_eap (iter, &method, &keyType, &identity, &identityPassword, &anonIdentity,
+						       &certPrivatePassword, &certPrivate, &certClient, &certCA, &version))
+			return false;
+	}
 
 	if ( !(version == IW_AUTH_WPA_VERSION_WPA || version == IW_AUTH_WPA_VERSION_WPA2 ) )
 		return false;
-	if ( !(method == EAP_PEAP || method == EAP_TLS || method == EAP_TTLS ) )
+	
+#ifdef HAVE_PHASE2_AUTH
+	eapMethod = (EAPMethod) NM_EAP_TO_EAP_METHOD(method);
+	eapPhaseTwo = (EAPPhaseTwo) NM_EAP_TO_PHASE2_METHOD(method);
+#else
+	eapMethod = (EAPMethod) method;
+	eapPhaseTwo = PHASE2_NONE;
+#endif
+
+	if ( !(eapMethod == EAP_PEAP || eapMethod == EAP_TLS || eapMethod == EAP_TTLS || eapMethod == EAP_LEAP) )
+		return false;
+	if ( !(eapPhaseTwo == PHASE2_NONE || eapPhaseTwo == PHASE2_PAP || eapPhaseTwo == PHASE2_MSCHAP || eapPhaseTwo == PHASE2_MSCHAPV2 || eapPhaseTwo == PHASE2_GTC) )
 		return false;
 
-	setMethod ((EAPMethod) method);
+	setMethod ( eapMethod );
+	setPhaseTwoAuth ( eapPhaseTwo );
 	setIdentity (identity);
 	setAnonIdentity (anonIdentity);
 	setCertPrivate (certPrivate);
@@ -665,7 +758,8 @@ EncryptionWPAEnterprise::deserialize( DBusMessageIter * iter, int we_cipher )
 bool
 EncryptionWPAEnterprise::isValid (const QString & essid)
 {
-	if (essid.length ())
+	bool hasSecret = (_secret[IdPasswordKey].length() > 0) || (_secret[CertPrivatePasswordKey].length() > 0);
+	if (essid.length () && hasSecret)
 		return true;
 	else
 		return false;
@@ -678,6 +772,7 @@ EncryptionWPAEnterprise::setDefaults (void)
 	setProtocol (WPA_EAP);
 	setVersion  (WPA1);
 	setMethod   (EAP_PEAP);
+	setPhaseTwoAuth (PHASE2_NONE);
 	setIdentity ("");
 	setAnonIdentity ("");
 	setCertPrivate ("");
@@ -807,6 +902,34 @@ EncryptionWPAEnterprise::getMethod (void)
 	return _method;
 }
 
+void
+EncryptionWPAEnterprise::setPhaseTwoAuth (EAPPhaseTwo auth)
+{
+	kdDebug() << k_funcinfo << " " << auth << endl;
+	_phaseTwo = auth;
+}
+
+EAPPhaseTwo
+EncryptionWPAEnterprise::getPhaseTwoAuth (void)
+{
+	kdDebug() << k_funcinfo << endl;
+	return _phaseTwo;
+}
+
+QString
+EncryptionWPAEnterprise::getLeapMethod (void)
+{
+	kdDebug() << k_funcinfo << endl;
+	return _leapmethod;
+}
+
+void
+EncryptionWPAEnterprise::setLeapMethod (const QString & leapmethod)
+{
+	kdDebug() << k_funcinfo << " " << leapmethod <<  endl;
+	_leapmethod = leapmethod;
+}
+
 EncryptionWPAEnterprise::EncryptionWPAEnterprise ()
 {
 	kdDebug() << k_funcinfo << endl;
@@ -829,15 +952,30 @@ void EncryptionWPAEnterprise::persist( KConfigBase * cfg, bool withKey ) const
 		cfg->writeEntry( "Method", "PEAP" );
 	} else if ( EAP_TLS == _method ) {
 		cfg->writeEntry( "Method", "TLS" );
+	} else if ( EAP_LEAP == _method ) {
+		cfg->writeEntry( "Method", "LEAP" );
 	} else {
 		cfg->writeEntry( "Method", "TTLS" );
 	}
 
+	if ( PHASE2_PAP == _phaseTwo ) {
+		cfg->writeEntry( "PhaseTwo", "PAP" );
+	} else if ( PHASE2_MSCHAP == _phaseTwo ) {
+		cfg->writeEntry( "PhaseTwo", "MSCHAP" );
+	} else if ( PHASE2_MSCHAPV2 == _phaseTwo ) {
+		cfg->writeEntry( "PhaseTwo", "MSCHAPV2" );
+	} else if ( PHASE2_GTC == _phaseTwo ) {
+		cfg->writeEntry( "PhaseTwo", "GTC" );
+	} else {
+		cfg->writeEntry( "PhaseTwo", "NONE");
+	}
+
 	cfg->writeEntry( "Identity", _identity );
 	cfg->writeEntry( "AnonIdentity", _anonIdentity );
 	cfg->writeEntry( "CertPrivate", _certPrivate );
 	cfg->writeEntry( "CertClient", _certClient );
 	cfg->writeEntry( "CertCA", _certCA );
+	cfg->writeEntry( "LeapMethod", _leapmethod);
 
 	if ( WPA_TKIP == _protocol )
 		cfg->writeEntry( "WPAProtocol", "TKIP" );
@@ -849,11 +987,14 @@ void EncryptionWPAEnterprise::persist( KConfigBase * cfg, bool withKey ) const
 	else
 		cfg->writeEntry( "WPAVersion", "WPA2" );
 
+	if ( withKey )
+		this->persistKey( );
+
 	cfg->writeEntry ( "Cipher", _we_cipher );
 	_dirty = false;
 }
 
-void EncryptionWPAEnterprise::restore( KConfigBase * cfg, double version, bool withKey)
+void EncryptionWPAEnterprise::restore( KConfigBase * cfg, const char* /*version*/, bool withKey)
 {
 	kdDebug() << k_funcinfo << endl;
 
@@ -877,17 +1018,38 @@ void EncryptionWPAEnterprise::restore( KConfigBase * cfg, double version, bool w
 		_method = EAP_PEAP;
 	} else if ( "TLS" == method ) {
 		_method = EAP_TLS;
+	} else if ( "LEAP" == method ) {
+		_method = EAP_LEAP;
 	} else {
 		_method = EAP_TTLS;
 	}
 
+	QString phaseTwo = cfg->readEntry( "PhaseTwo" );
+	if ( "PAP" == phaseTwo ) {
+		_phaseTwo = PHASE2_PAP;
+	} else if ( "MSCHAP" == phaseTwo ) {
+		_phaseTwo = PHASE2_MSCHAP;
+	} else if ( "MSCHAPV2" == phaseTwo ) {
+		_phaseTwo = PHASE2_MSCHAPV2;
+	} else if ( "GTC" == phaseTwo ) {
+		_phaseTwo = PHASE2_GTC;
+	} else {
+		_phaseTwo = PHASE2_NONE;
+	}
+
 	_identity = cfg->readEntry( "Identity" );
 	_anonIdentity = cfg->readEntry( "AnonIdentity" );
 	_certPrivate = cfg->readEntry( "CertPrivate" );
 	_certClient = cfg->readEntry( "CertClient" );
 	_certCA = cfg->readEntry( "CertCA" );
+
+	_leapmethod = cfg->readEntry( "LeapMethod" );
+
 	// _we_cipher is not used in WPAEnterprise, so no verison check required
 	_we_cipher = cfg->readNumEntry( "Cipher", -1 );
 
+	if ( withKey && hasStoredKey() )
+		this->restoreKey( );
+
 	_dirty = false;
 }
