@@ -7,6 +7,7 @@
  * Author: Timo Hoenig        <thoenig@suse.de>, <thoenig@nouse.net>
  *         Will Stephenson    <wstephenson@suse.de>, <wstephenson@kde.org>
  *         Valentine Sinitsyn <e_val@inbox.ru>
+ *         Helmut Schaa       <hschaa@suse.de>, <helmut.schaa@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,11 +47,16 @@ class WirelessDialog;
 #include <klocale.h>
 #include <kstdguiitem.h>
 #include <khelpmenu.h>
-
+#include <kprocess.h>
 #include <kiconloader.h>
+#include <kconfig.h>
+#include <kmessagebox.h>
 
 #include "knetworkmanager-ui-networklistview.h"
 #include "knetworkmanager-storage.h"
+#include "knetworkmanager-vpnconnectionsdialog.h"
+#include "knetworkmanager-connectioninfodialog.h"
+#include "knetworkmanager-settings.h"
 
 #define KDED_NETWORK_NAME "NMNetwork"
 
@@ -173,11 +179,12 @@ Tray::updateTooltip ()
 					Network* net = store->getActiveNetwork (dev);
 					_tooltip = i18n ("Wireless ethernet connection active: ") + dev->getInterface ();
 					if (net) {
-						_tooltip += " (" + i18n ("active network: ") + net->getEssid () + ")";
+						_tooltip += "\n" + i18n ("active network: ") + net->getEssid ();
 					} else {
-						_tooltip += " " + i18n ("Warning: No active wireless network found.");
+						_tooltip += "\n" + i18n ("Warning: No active wireless network found.");
 					}
 				}
+			_tooltip += "\n" + i18n ("IP: ") +dev->getIPv4Address ();
 			}
 		} else if (state->isSleeping ()) {
 			_tooltip = i18n ("Offline Mode is currently active");
@@ -218,29 +225,18 @@ Tray::drawContents (QPainter *p)
 {
 	QRect r = contentsRect();
 
-	if (m_animated) {
-		if (!testWFlags(Qt::WNoAutoErase)) { // We are rendering a first frame - grab the background
-			setWFlags(Qt::WNoAutoErase);
-			erase(r); // Be sure we grab the background, not anything else
-			m_background = QPixmap::grabWindow( this->winId(), r.x(), r.y(), r.width(), r.height() );
-		}
-
-		QPixmap backBuffer(m_background);
-		QPainter back(&backBuffer);
-
-		back.translate(-r.x(), -r.y()); // KSystemTray::drawContents() draws inside contentsRect() only, but we are already inside.
-		KSystemTray::drawContents(&back);  
-		bitBlt(this, r.topLeft(), &backBuffer);
-	} else {
-		if (testWFlags(Qt::WNoAutoErase)) { // First non-animated frame - revert to normal rendering
-			clearWFlags(Qt::WNoAutoErase);
-			erase(r);
-		}
-
+	if (movie())
 		KSystemTray::drawContents(p);
+	else
+	{
+		KSystemTray::drawContents(p);
+		// draw overlay image
+		if (!m_overlay.isNull())
+			p->drawPixmap(2,r.height()/2-2,m_overlay);
 	}
 }
 
+
 void
 Tray::connectHiddenNetwork ()
 {
@@ -252,7 +248,18 @@ Tray::connectHiddenNetwork ()
 void
 Tray::configureVPN ()
 {
-	emit configureVPNConnections ();
+	VPNConnectionsDialog* mvc;
+	mvc = dynamic_cast<VPNConnectionsDialog*>(this->child("VPNConnectionsDialog", "VPNConnectionsDialog"));
+	if (!mvc)
+		mvc = new VPNConnectionsDialog(_ctx, this, "VPNConnectionsDialog", FALSE, Qt::WDestructiveClose);
+	
+	// we want to notify NM when connections are changed
+	connect(mvc, SIGNAL(done()), _ctx->getVPN(), SLOT(updateVPNConnections()));
+	
+	// show the new Dialog
+	mvc->show();
+	mvc->raise();
+	mvc->setActiveWindow();
 }
 
 void
@@ -261,12 +268,34 @@ Tray::disconnectVPN ()
 	emit disconnectVPNConnection ();
 }
 
+void
+Tray::slotQuit()
+{
+	KConfig * config = KGlobal::config();
+	config->setGroup("Notification Messages");
+	if (config->readEntry("AutostartDontAskAgain").isEmpty()) {
+		int answer = KMessageBox::questionYesNo (
+		  0,
+		  i18n ("Start KNetworkManager automatically when you log in?"),
+		  i18n ("Question"),
+		  i18n ("Start Automatically"),
+		  i18n ("Do Not Start"),
+		  "AutostartDontAskAgain");
+
+		config->setGroup ("General");
+		config->writeEntry ("Autostart", answer == KMessageBox::Yes);
+		config->sync ();
+	}
+}
+
 void
 Tray::configureDialUp ()
 {
 #ifdef KNETWORKMANAGER_DIALUP_CONFIG
 	KProcess* dialupConfig = new KProcess ();
-	*dialupConfig << KNETWORKMANAGER_DIALUP_CONFIG;
+	QStringList command = QStringList::split (" ", KNETWORKMANAGER_DIALUP_CONFIG, false);
+	for (QStringList::Iterator it = command.begin (); it != command.end (); ++it)
+		*dialupConfig << (*it);
 	dialupConfig->start ();
 #endif
 }
@@ -355,11 +384,13 @@ Tray::showNetworksDialog()
 void
 Tray::itemHighlighted (int id)
 {
-	Device* dev = _deviceMap[id];
-	Network* net = _networkMap[id];
+	Device* dev = _ctx->getDeviceStore()->getDevice(_deviceMap[id]);
+	Network* net = NULL;
+	if (_networkMap[id])
+		net = dev->getNetwork(_networkMap[id]);
 	QString tooltip;
 
-	if (dev) {
+	if (dev && !net) {
 		int speed = dev->getSpeed ();
 
 		tooltip = dev->getVendor () + " "	+ dev->getProduct () 								+ "\n";
@@ -380,6 +411,7 @@ Tray::itemHighlighted (int id)
                 if (dev) {
 			int     signalStrength = net->getStrength () ? net->getStrength () : dev->getStrength ();
 			int     speed          = net->getRate () / 1024;
+			if (speed == 0) speed = dev->getSpeed();
 			double  frequency      = net->getFrequency ();
 
 			tooltip = dev->getVendor () + " "      + dev->getProduct ()								+ "\n";
@@ -423,20 +455,23 @@ Tray::itemHighlighted (int id)
 void
 Tray::itemActivated (int id)
 {
-	Device* dev = _deviceMap[id];
-	Network* net = _networkMap[id];
+	Device* dev = _ctx->getDeviceStore()->getDevice(_deviceMap[id]);
+	Network* net = NULL;
+	if (_networkMap[id])
+		net = dev->getNetwork(_networkMap[id]);
 	ActivationStageNotifyNetwork* notify;
 
 	if (!dev && !net) {
 		return;
 	}
 
-	if (dev) {
+	if (dev && !net) {
 		QString iface = dev->getInterface ();
 		emit activateDevice (dev);
 	} else if (net) {
 		if (!dev) {
-			dev = _ctx->getDeviceStore ()->getDevice (net);
+			if (NULL == (dev = _ctx->getDeviceStore ()->getDevice (net)))
+				return;
 		}
 
 		emit activateNetwork (net, dev);
@@ -445,6 +480,8 @@ Tray::itemActivated (int id)
 	/* destroy an activation stage notify dialogs and create a new one */
 	emit destroyActivationStage ();
 	notify = new ActivationStageNotifyNetwork ( net ? net->getEssid() : QString::null, this, "ActivationStageNotify", false, 0, _ctx, dev );
+	notify->setAnchor(_anchor);
+	notify->show();
 }
 
 void
@@ -455,12 +492,13 @@ Tray::vpnItemActivated (int id)
 
 	if (!vpnConnection)
 		return;
-	
-	emit activateVPNConnection (vpnConnection);
 
 	/* destroy an activation stage notify dialogs and create a new one */
 	emit destroyActivationStage ();
 	notify = new ActivationStageNotifyVPN (this, "ActivationStageNotify", false, 0, _ctx, vpnConnection);
+	notify->setAnchor(_anchor);
+	notify->show();
+	emit activateVPNConnection (vpnConnection);
 }
 
 void
@@ -470,6 +508,11 @@ Tray::addVPNConnection (VPNConnection* vpnConnection, bool state)
 	QString item = QString::null;
 	QString prefix = QString::null;
 	QString postfix = QString::null;
+	QString icon("encrypted");
+
+	VPNService* service = vpnConnection->getVPNService();
+	if (service)
+		icon = service->getIcon();
 
 	NMVPNActStage stage = vpnConnection->getActivationStage ();
 
@@ -497,7 +540,7 @@ Tray::addVPNConnection (VPNConnection* vpnConnection, bool state)
 	if (postfix != QString::null)
 		item += " (" + postfix + ")";
 
-	id = _vpnMenu->insertItem (SmallIcon ("encrypted", QIconSet::Automatic), item, -1, -1);
+	id = _vpnMenu->insertItem (SmallIcon (icon, QIconSet::Automatic), item, -1, -1);
 	_vpnMenu->setItemEnabled (id, state);
 
 	if (stage == NM_VPN_ACT_STAGE_ACTIVATED)
@@ -563,7 +606,8 @@ Tray::addWirelessNetwork (Device* dev, Network* net)
 
 	contextMenu ()->setItemChecked (id, online && net->isActive () && dev->isActive ());
 
-	_networkMap[id] = net;
+	_networkMap[id] = net->getObjectPath();
+	_deviceMap[id] = dev->getObjectPath();
 }
 
 void
@@ -578,7 +622,7 @@ Tray::addWiredDevice (Device* dev, const QString & descr)
 	contextMenu ()->setItemChecked (id, online && dev->isActive ());
 	contextMenu ()->setItemEnabled (id, dev->getLinkActive ());
 
-	_deviceMap[id] = dev;
+	_deviceMap[id] = dev->getObjectPath();
 }
 
 void
@@ -648,6 +692,16 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 		offlineMode->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
 		int id = contextMenu ()->insertItem (offlineMode);
 		menu->setItemEnabled (id, false);
+	} else if (state->isWaitingForKey()) {
+		QLabel* awaitingKey = new QLabel (i18n ("<b>Waiting for key from KWallet.<b>"), 0);
+
+		awaitingKey->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
+		int id = contextMenu ()->insertItem (awaitingKey);
+		menu->setItemEnabled (id, false);
+		
+		id = contextMenu()->insertItem(SmallIcon ("cancel",  QIconSet::Automatic), i18n ("Cancel key request"), KNetworkManagerStorage::getInstance(), SLOT(slotCancelAllCredentialRequests()));
+		menu->insertSeparator();
+
 	} else {
 		/* add wired devices and a separator */
 		if (stat->wiredDevices) {
@@ -712,13 +766,6 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 			}
 		}
 
-#ifdef KNETWORKMANAGER_DIALUP_CONFIG
-		if (stat->dialUpConnections)
-			_dialUpMenu->insertSeparator ();
-		KAction* configureDialUpAction = actionCollection ()->action ("configure_dialup");
-		configureDialUpAction->plug (_dialUpMenu);
-#endif
-
 #ifndef KNETWORKMANAGER_DIALUP_CONFIG
 		if (stat->dialUpConnections)
 			menu->insertItem (SmallIcon ("modem",  QIconSet::Automatic), i18n ("Dial-Up Connections"), _dialUpMenu, -1);
@@ -736,8 +783,11 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 				/* enable items of VPN connections if (a) no other VPN connection is active
 				 * and (b) we're currently connected to the internet
 				 */
-				addVPNConnection (*i, !isActive && state->isConnected ());
-				vpnConnectionFound = true;
+				if ((*i)->isConfirmedByNM())
+				{
+					addVPNConnection (*i, !isActive && state->isConnected ());
+					vpnConnectionFound = true;
+				}
 			}
 
 			if (vpnConnectionFound)
@@ -747,16 +797,13 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 			disconnectVPNAction->setEnabled (isActive);
 			disconnectVPNAction->plug (_vpnMenu);
 
-			KAction* configureVPNAction = actionCollection ()->action ("configure_vpn");
-			configureVPNAction->plug (_vpnMenu);
-
 			menu->insertItem (SmallIcon ("encrypted",  QIconSet::Automatic), i18n ("VPN Connections"), _vpnMenu, -1);
 			separated = false;
 		}
 	}
 
 	/* build-up the ``Options'' sub menu if NM is running */
-	if (state->isNetworkManagerRunning ()) {
+	if (state->isNetworkManagerRunning () && !state->isWaitingForKey()) {
 		if (!separated)
 		    menu->insertSeparator ();
 		KAction* switchWirelessOnAction  = actionCollection ()->action ("switch_wireless_on");
@@ -776,12 +823,9 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 		switchOffAction->plug (_optionsMenu);
 
 		_optionsMenu->insertSeparator ();
+		actionCollection()->action("show_connection_info")->plug(_optionsMenu);
+		actionCollection()->action("show_settings_dialog")->plug(_optionsMenu);
 
-		showNetworksDialogAction->plug (_optionsMenu);
-		
-		KAction *configureNotificationsAction = KStdAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
-		configureNotificationsAction->plug(_optionsMenu);
-		
 		menu->insertItem (SmallIcon ("configure",  QIconSet::Automatic), i18n ("Options"), _optionsMenu, -1);
 	}
 
@@ -799,37 +843,50 @@ Tray::contextMenuAboutToShow (KPopupMenu* menu)
 	}
 }
 
-void Tray::mouseMoveEvent (QMouseEvent * /* e */)
+void Tray::mouseMoveEvent (QMouseEvent * e)
 {
 	userInteraction ();
+	_anchor = e->globalPos();
 }
 
 void Tray::mousePressEvent( QMouseEvent *e )
 {
-	if (e->button() == RightButton || e->button () == LeftButton) {
+	if (e->button() == RightButton) {
 		contextMenuAboutToShow (contextMenu());
 	 	contextMenu ()->popup (e->globalPos());
 	}
+	else if (e->button () == LeftButton) {
+		ConnectionInfoDialog* dlg = NULL;
+		dlg = dynamic_cast<ConnectionInfoDialog*>(this->child("ConnectionInfoDialog", "ConnectionInfoDialog"));
+		if (dlg)
+			if (dlg->isHidden())
+				dlg->show();
+			else
+				dlg->close();
+		else
+			slotShowConnectionInfo();
+	}
+}
+
+void Tray::vpnConnectionStateChanged(bool connected)
+{
+	if (connected) {
+		QRect r = contentsRect();
+		m_overlay = SmallIcon("encrypted", r.width()*1/2);
+	}
+	else
+		m_overlay = QPixmap();
+	repaint();
 }
 
 void Tray::slotStateChanged()
 {
 	QPixmap pm = updateForState( _ctx->getState( )->getConnectionState( ) );
 
-	if ( m_animated )
-		m_iconTimer.start( 100 );
-	else if ( m_iconTimer.isActive() )
-		m_iconTimer.stop();
-	if ( m_animated || pm.serialNumber() != pixmap()->serialNumber() )
-		setPixmap( pm );
-}
 
-void
-Tray::slotIconTimer()
-{
-	m_step++;
-	if ( m_animated && _ctx->getState( )->getConnectionState() == NM_STATE_CONNECTING )
-		setPixmap( pixmapForStage() );
+	if (!pm.isNull() && ((movie() != 0) || (pm.serialNumber() != pixmap()->serialNumber()))) {
+		setPixmap( pm );
+	}
 }
 
 void
@@ -837,15 +894,6 @@ Tray::loadIcons()
 {
 	// load all the connecting icons and the static (connected, disconnected etc icons)
 	KGlobal::iconLoader()->addAppDir("knetworkmanager");
-	QString templateName;
-	for ( int i = 0; i < NUM_CONNECTING_STAGES; i++ )
-	{
-		for (int j = 0; j < NUM_CONNECTING_FRAMES; j ++ )
-		{
-			templateName.sprintf( "nm_stage%02i_connecting%02i", i+1, j+1 );
-			m_pixmaps[i][j] = SmallIcon( templateName, 22 );
-		}
-	}
 	m_disconnected = SmallIcon( "nm_no_connection", 22 );
 	m_wired = SmallIcon( "nm_device_wired", 22 );
 	m_dialup = SmallIcon ("modem", 22);
@@ -891,15 +939,10 @@ Tray::updateForState( NMState state )
 			else 
 				pixmap = m_wired;
 
-			m_animated = false;
-			
 			break;
 		case NM_STATE_CONNECTING:
 			if ( dev )
-			{
 				pixmap = pixmapForStage();
-				m_animated = true;
-			}
 			else 
 				pixmap = m_disconnected;
 			break;
@@ -911,7 +954,6 @@ Tray::updateForState( NMState state )
 			else
 				pixmap = m_disconnected;
 
-			m_animated = false;
 			break;
 	}
 
@@ -950,9 +992,21 @@ QPixmap Tray::pixmapForStage()
 		}
 		if (connectingStage >= 0 && connectingStage < NUM_CONNECTING_STAGES)
 		{
-			if (m_step >= NUM_CONNECTING_FRAMES)
-				m_step = 0;
-			pixmap = m_pixmaps [connectingStage][m_step];
+			QString iconName;
+			int iconState = connectingStage +1;
+			int frame = -1;
+
+			iconName.sprintf( "nm_stage%02i_connecting", iconState );
+			if (movie()) {
+				frame = movie()->frameNumber();
+			}
+
+			setMovie(QMovie( KGlobal::iconLoader()->moviePath(iconName, KIcon::Panel) ));
+
+			if (frame > 0) {
+				movie()->step(frame);
+				movie()->unpause();
+			}
 		}
 	} else {
 		pixmap = m_disconnected;
@@ -1012,14 +1066,46 @@ void Tray::slotNetworkDisappeared( Network * net )
 	KNotifyClient::event( winId(), "knm-nm-network-gone", i18n("Wireless network %1 disappeared").arg( net->getEssid() ) );
 }
 
+QPoint
+Tray::getAnchor() const
+{
+	return _anchor;
+}
+
 void
 Tray::push (KNetworkManager* ctx)
 {
 	_ctx = ctx;
 }
 
+void
+Tray::slotShowConnectionInfo()
+{
+	ConnectionInfoDialog* dlg = NULL;
+	dlg = dynamic_cast<ConnectionInfoDialog*>(this->child("ConnectionInfoDialog", "ConnectionInfoDialog"));
+	if (!dlg)
+		dlg = new ConnectionInfoDialog(_ctx, this, 
+                        "ConnectionInfoDialog", Qt::WDestructiveClose);
 
-Tray::Tray () : KSystemTray (), m_step( 0 )
+	// show the dialog
+	dlg->show();
+	dlg->raise();
+	dlg->setActiveWindow();
+}
+
+void
+Tray::slotShowSettingsDialog()
+{
+	SettingsDialog* dlg = NULL;
+	dlg = dynamic_cast<SettingsDialog*>(this->child("SettingsDialog", "SettingsDialog"));
+	if (!dlg)
+		dlg = new SettingsDialog(_ctx, this, "SettingsDialog", Qt::WDestructiveClose, false, "Settings");
+	dlg->show();
+	dlg->raise();
+	dlg->setActiveWindow();
+}
+
+Tray::Tray () : KSystemTray () 
 {
 /*	QMovie movie = KGlobal::iconLoader()->loadMovie("nm_detect", KIcon::Panel );
 // 	movie.setSpeed( 3 );
@@ -1076,6 +1162,13 @@ Tray::Tray () : KSystemTray (), m_step( 0 )
 	showNetworksDialogAction = new KAction (i18n ("Show Networks..."),
 						   0, 0,
 						   this, SLOT (showNetworksDialog ()), actionCollection (), "show_networks_dialog");
+	showConnectionInfo = new KAction(i18n ("Show Connection Information..."),
+						   SmallIcon ("info", QIconSet::Automatic), 0,
+						   this, SLOT(slotShowConnectionInfo()), actionCollection(), "show_connection_info");
+
+	showSettingsDialog = new KAction(i18n ("Configure..."),
+						   SmallIcon ("package_settings", QIconSet::Automatic), 0,
+						   this, SLOT(slotShowSettingsDialog()), actionCollection(), "show_settings_dialog");
 
 	connect (this->contextMenu (), SIGNAL (activated (int)),
 		 this,                 SLOT   (itemActivated (int)));
@@ -1089,9 +1182,9 @@ Tray::Tray () : KSystemTray (), m_step( 0 )
 	connect (this->_dialUpMenu, SIGNAL (activated (int)),
 		 this,              SLOT   (dialUpItemActivated (int)));
 
-	loadIcons();
+	connect (this, SIGNAL(quitSelected()), this, SLOT(slotQuit()));
 
-	connect ( &m_iconTimer, SIGNAL( timeout() ), SLOT( slotIconTimer() ) );
+	loadIcons();
 }
 
 Tray::~Tray ()
